------------------------------------------------------------------------------
--
--  Xilinx, Inc. 2006                 www.xilinx.com
--
--  XAPP 486 - 7:1 LVDS in Spartan3E Devices
--
------------------------------------------------------------------------------
--
--  File name :       top4_tx.vhd
--
--  Description :     Example top level module for using a 4-bit transmitter in Spartan 3E
--
--  Date - revision : October 16th 2006 - v 1.4
--			
--			Version 1.4 : 	Brings the DDR registers to the top level and no
--					longer uses 'C0' alignment
--

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all ;

library unisim ;
use unisim.vcomponents.all ;



entity top4_tx is port (
	clkin			: in std_logic ; 			-- clock in
	datain			: in std_logic_vector(27 downto 0);	-- 28 bit data in
	rstin			: in std_logic;				-- reset (active low)
	dataouta_p, dataouta_n	: out std_logic_vector(3 downto 0);	-- lvds data outputs
	clkouta1_p, clkouta1_n	: out std_logic) ;			-- lvds clock output
end top4_tx;

architecture arch_top4_tx of top4_tx is


component serdes_4b_7to1_wrapper port (
	clk 			: in std_logic ;			-- clock
	clkx3p5 		: in std_logic ;			-- 3.5 times clock
	clkx3p5not		: in std_logic ;			-- not 3.5 times clock
	datain 			: in std_logic_vector(27 downto 0) ;	-- input data
	rst 			: in std_logic ;			-- reset
	dataout			: out std_logic_vector(7 downto 0) ;	-- output data
	clkout			: out std_logic_vector(1 downto 0)) ;	-- output clock (1x)
end component;



signal low 		: std_logic ;				-- logic '0' 
signal rst 		: std_logic ;				-- reset signal
signal inclk 		: std_logic ;				-- 
signal inclknot		: std_logic ;				-- 
signal clk 		: std_logic ;				-- main clock from DCM
signal clknot 		: std_logic ;				-- inverted main clock from DCM
signal clkint		: std_logic ;				-- clock input from pin
signal clkdcm 		: std_logic ;				-- clock from dcm
signal clkx3p5 		: std_logic ;				-- 3.5x clock for transmitter
signal clkx3p5dcm 	: std_logic ;				-- 3.5x clock from dcm
signal clkx3p5notdcm 	: std_logic ;				-- not 3.5x clock from dcm
signal outdata 		: std_logic_vector(7 downto 0) ;	-- output data lines
signal oclkinta		: std_logic_vector(1 downto 0) ;
signal clkoutint 	: std_logic ;	          		-- forwarded output clock
signal clkoutaint 	: std_logic ;	          		-- forwarded output clock from macro 3:4 or 4:3 duty cycle
signal clkoutbint 	: std_logic ;	          		-- forwarded output clock using DCM clk0 - 50% output duty cycle 
signal clkoutcint 	: std_logic ;	          		-- forwarded output clock just using BUFG - output duty cycle = input duty cycle
signal clkoutdint 	: std_logic ;	          		-- output clock being used to monitor CLKFX and CLKFX180
signal clk_lckd 	: std_logic ;				-- clock locked
signal not_clk_lckd 	: std_logic ;				-- not clock locked
signal txdata 		: std_logic_vector(27 downto 0) ;	-- data for transmission
signal clkx3p5not 	: std_logic ;				-- inverted 3.5x clock
signal rst_clk 		: std_logic ;				-- reset syncced to main clock
signal tx_output_reg  	: std_logic_vector(3 downto 0) ;
signal tx_output_fix  	: std_logic_vector(7 downto 0) ;


constant TX_SWAP_MASK  	: std_logic_vector(3 downto 0) := "0000" ;
	
begin

low 	 <= '0' ;
rst 	 <= not rstin ; 					-- reset is active low
clknot   <= not clk ;
inclknot <= not inclk ;

dcm_clk : dcm_sp	
generic map (
	DESKEW_ADJUST 		=> "0",
	CLKIN_PERIOD 		=> 12.0,
	CLKFX_MULTIPLY 		=> 7,
	CLKFX_DIVIDE		=> 2)
port map (
	clkin   	=> clkint,
	CLKFB   	=> clk,
	DSSEN 		=> low,
	PSINCDEC	=> low,
	PSEN 		=> low,
	PSCLK 		=> low,
	RST     	=> rst,
	CLK0    	=> clkdcm,
	CLK90   	=> open,
	CLK180  	=> open,
	CLK270  	=> open,
	CLK2X   	=> open,
	CLK2X180	=> open,
	CLKDV   	=> open,
	CLKFX   	=> clkx3p5dcm,
	CLKFX180	=> clkx3p5notdcm,
	LOCKED  	=> clk_lckd,
	PSDONE  	=> open,
	STATUS  	=> open);
	
clk_ibuf 	: ibufg_lvcmos25     port map(i => clkin, 	o => clkint );


inclk_bufg 	: bufg 	port map(i => clkint, 		o => inclk ) ; --port map(i => clkint, 		o => inclk ) ;
clk_bufg 	: bufg 	port map(i => clkdcm, 		o => clk ) ;
clkx3p5_bufg 	: bufg 	port map(i => clkx3p5dcm, 	o => clkx3p5 ) ;
clkx3p5not_bufg : bufg 	port map(i => clkx3p5notdcm, 	o => clkx3p5not ) ;
--clkx3p5not <= not clkx3p5 ;

loop0 : for i in 0 to 3 generate 
obuf_d 	: obufds generic map (IOSTANDARD => "LVDS_25") 
	         port map    (i => tx_output_reg(i), o => dataouta_p(i), oB => dataouta_n(i));
fd_d  	: oddr2  generic map (DDR_ALIGNMENT => "NONE")
                 port map (c0 => clkx3p5, c1 => clkx3p5not, d0 => tx_output_fix(i+4), d1 => tx_output_fix(i), ce => '1', r => '0', s => '0', q => tx_output_reg(i));
tx_output_fix(i)   <= outdata(i) xor TX_SWAP_MASK(i) ;
tx_output_fix(i+4) <= outdata(i+4) xor TX_SWAP_MASK(i) ;
end generate ;

ca_ddr_reg : oddr2 
generic map 	(ddr_alignment => "NONE")
port map 	(d0 => oclkinta(1), d1 => oclkinta(0), c0 => clkx3p5, c1 => clkx3p5not, ce => '1', r => '0', s => '0', q => clkoutaint);
	
cb_ddr_reg : oddr2 
generic map 	(ddr_alignment => "NONE")
port map 	(d0 => '1', d1 => '0', c0 => clk,     c1 => clknot,     ce => '1', r => '0', s => '0', q => clkoutbint);

cc_ddr_reg : oddr2 
generic map 	(ddr_alignment => "NONE")
port map 	(d0 => '1', d1 => '0', c0 => inclk,   c1 => inclknot,   ce => '1', r => '0', s => '0', q => clkoutcint);

cd_ddr_reg : oddr2 
generic map 	(ddr_alignment => "NONE")
port map 	(d0 => '1', d1 => '0', c0 => clkx3p5, c1 => clkx3p5not, ce => '1', r => '0', s => '0', q => clkoutdint);
		
clkoutint <= clkoutaint	;	-- use this line for 3:4 macro generated forwarded clock
--clkoutint <= clkoutbint	;	-- use this line for 50:50 forwarded clock using DCM
--clkoutint <= clkoutcint	;	-- use this line for input to output forwarded clock
--clkoutint <= clkoutdint	;	-- use this line to monitor CLKFX and CLKFX180 for test purposes

lvds_clka_obuf  : obufds generic map (IOSTANDARD => "LVDS_25") port map(i => clkoutint, 	o => clkouta1_p,  	oB => clkouta1_n );
	
tx0 : serdes_4b_7to1_wrapper port map(
	clk		=> clk,
	datain 		=> txdata,
	rst   		=> rst_clk,
	clkx3p5   	=> clkx3p5,
	clkx3p5not	=> clkx3p5not,
	dataout		=> outdata,
	clkout		=> oclkinta );	-- clock output

process (clk, rst_clk)
begin
if rst_clk = '1' then
	txdata <= (others => '0') ;
elsif clk'event and clk = '1' then
	txdata <= datain ;
end if ;
end process ;

not_clk_lckd <= not clk_lckd ;

-- generate a registered reset signal for the tx clock
fd_rst_clk : fdp port map (d => not_clk_lckd, c => clk, pre => rst, q => rst_clk) ;

end arch_top4_tx;


